package org.ten60.netkernel.test.app;

import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.representation.*;
import org.netkernel.layer0.representation.impl.*;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.request.IRequest;
import org.w3c.dom.Document;

public class TestExecAccessor extends StandardAccessorImpl
{
	public TestExecAccessor()
	{	declareThreadSafe();
	}
	
	public void onSource(INKFRequestContext context) throws Exception
	{
		String testid=context.getThisRequest().getArgumentValue("id");
		boolean all=testid.equals("all");
		String format=context.getThisRequest().getArgumentValue("format");
		context.setCWU(null);
		INKFResponse resp=null;

		if(all)
		{	INKFRequest req=context.createRequest("active:testTree");
			req.setHeader(IRequest.HEADER_FORGET_DEPENDENCIES,Boolean.TRUE);
			req.setRepresentationClass(IHDSNode.class);
			IHDSNode testlist=(IHDSNode)context.issueRequest(req);
			int testTotal =0, testRun=0, testSuccess=0, testFailException=0, testFailAssert=0,	testExecutionTime=0, testTotalTime=0;
			String testDate="";
			HDSBuilder b=new HDSBuilder();
			b.pushNode("testResults");
			for(IHDSNode space : testlist.getNodes("/spaces/space"))
			{	String spaceid=(String)space.getFirstValue("id").toString().trim();
				String version=(String)space.getFirstValue("version").toString().trim();
				
				for(IHDSNode test : space.getNodes("tests/test"))
				{	b.pushNode("test");
					b.addNode("id", test.getFirstValue("id"));
					req=context.createRequest("active:test");
					req.setHeader(IRequest.HEADER_FORGET_DEPENDENCIES,Boolean.TRUE);
					req.addArgument("space", spaceid);
					req.addArgument("version", version);
					req.addArgument("tests", (String)test.getFirstValue("uri").toString().trim());
					req.setRepresentationClass(IHDSNode.class);
					IHDSNode result=(IHDSNode)context.issueRequest(req);
					testTotal+=Integer.valueOf((String)result.getFirstValue("/testlist/results/testTotal"));
					testRun+=Integer.valueOf((String)result.getFirstValue("/testlist/results/testRun"));
					testSuccess+=Integer.valueOf((String)result.getFirstValue("/testlist/results/testSuccess"));
					testFailException+=Integer.valueOf((String)result.getFirstValue("/testlist/results/testFailException"));
					testFailAssert+=Integer.valueOf((String)result.getFirstValue("/testlist/results/testFailAssert"));
					testExecutionTime+=Integer.valueOf((String)result.getFirstValue("/testlist/results/testExecutionTime"));
					testTotalTime+=Integer.valueOf((String)result.getFirstValue("/testlist/results/testTotalTime"));
					if (testDate.equals("")) testDate=(String)result.getFirstValue("/testlist/results/testDate");
					b.importNode(result.getFirstNode("/testlist/results"));
					b.popNode();
				}
			}
				b.pushNode("totalResults");
					b.addNode("testDate", testDate);
					b.addNode("testTotal", testTotal);
					b.addNode("testRun", testRun);
					b.addNode("testSuccess", testSuccess);
					b.addNode("testFailException", testFailException);
					b.addNode("testFailAssert", testFailAssert);
					b.addNode("testExecutionTime", testExecutionTime);
					b.addNode("testTotalTime", testTotalTime);
				b.popNode();
			b.popNode();
			Object result=context.transrept(b.getRoot(), Document.class);
			if(format.equals("html"))
			{	req=context.createRequest("active:xslt");
				req.addArgumentByValue("operand", result);
				req.addArgument("operator","res:/org/ten60/netkernel/test/app/style-all-results.xsl");
				req.setRepresentationClass(String.class);
				result=(String)context.issueRequest(req);
				resp=context.createResponseFrom(result);
				resp.setHeader("WrappedControlPanel", TestViewAccessor.TITLE+"Execute All");
			}
			else
			{	resp=context.createResponseFrom(result);
			}
		}
		else
		{
			INKFRequest req=context.createRequest("active:testEntry");
			req.addArgument("id", testid);
			IHDSNode test=(IHDSNode)context.issueRequest(req);
			
			String index=null;
			req=context.createRequest("active:test");
			req.addArgument("space", (String)test.getFirstValue("test/space").toString().trim());
			req.addArgument("version", (String)test.getFirstValue("test/version").toString().trim());
			req.addArgument("tests", (String)test.getFirstValue("test/uri").toString().trim());
			req.setHeader(INKFRequest.HEADER_FORGET_DEPENDENCIES, Boolean.TRUE);
			if(context.exists("httpRequest:/param/index"))
			{	index=(String)context.source("httpRequest:/param/index");
				req.addArgument("index", index);	
				INKFResponseReadOnly response=context.issueRequestForResponse(req);
				Object result=response.getRepresentation();
				if(! (result instanceof  IReadableBinaryStreamRepresentation))
				{	try
					{	response=context.transreptForResponse(result, IReadableBinaryStreamRepresentation.class);
						resp=context.createResponseFrom(response);
					}
					catch(Exception e)
					{	if (result!=null)
						{	result=result.toString();
						}
						else
						{	result="<null>";
						}
						resp=context.createResponseFrom(result);
						resp.setMimeType("text/plain");
					}
				}
				else
				{	resp=context.createResponseFrom(response);
				}
			}
			else
			{	req.setRepresentationClass(org.w3c.dom.Document.class);
				Object result=context.issueRequest(req);
				if(format.equals("html"))
				{	req=context.createRequest("active:xslt");
					req.addArgumentByValue("operand", result);
					req.addArgument("operator","res:/org/ten60/netkernel/test/app/style-results.xsl");
					req.addArgumentByValue("test", test);
					req.setRepresentationClass(String.class);
					result=context.issueRequest(req);
					resp=context.createResponseFrom(result);
					String name=test.getFirstValue("/test/name").toString();
					resp.setHeader("WrappedControlPanel", TestViewAccessor.TITLE+name);
					String subtitle="from "+test.getFirstValue("/test/space").toString()+" v"+test.getFirstValue("/test/version").toString();
					resp.setHeader("WrappedControlPanelSubtitle", subtitle);
				}
				else
				{	resp=context.createResponseFrom(result);
					resp.setMimeType("text/xml");
				}
			}
		}
		//Expire response
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
	}
	
	
}
