<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:nk="http://netkernel.org" xmlns:xrl="http://netkernel.org/xrl">
    <xsl:output method="xml"/>
	<xsl:param name="test"/>
	
	<xsl:template match="/testlist">
		<div>
			<table>
			<tr>
			<td>
			<div>
				<input type="button" value="exec all" onclick="parent.location='/test/exec/html/{$test/test/id}'" class="btn"/>
				<input type="button" value="view xml testlist" onclick="parent.location='/test/view/xml/{$test/test/id}'" class="btn"/>
			</div>
			</td>
			</tr>
			<tr>
			<td>
			<div style="margin-bottom: 10px;">
				<xsl:apply-templates select="results"/>
			</div>
			
			<table class="data-table">
				<tr class="vis-title">

				<td></td>
				<td>Test</td>
				<td>Time</td>
				<td>Status</td>
				<td/>
				</tr>
				<xsl:for-each select="test | notest">
						 <xsl:variable name="class">
							<xsl:choose>
								<xsl:when test="name()='notest'">
									<xsl:text>xunit-notest</xsl:text>
								</xsl:when>
							    <xsl:when test="position() mod 2 = 0">
								    <xsl:text>highrow</xsl:text>
							    </xsl:when>
							    <xsl:otherwise>
								    <xsl:text>highrow</xsl:text>
							    </xsl:otherwise>   
							</xsl:choose> 
					    </xsl:variable>
					<tr class="{$class}">
					    <td style="text-align: right;"><xsl:value-of select="position()"/></td>
						<td>
						<xsl:choose>
							<xsl:when test="@name">
								<xsl:value-of select="@name"/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="request/identifier"/>
							</xsl:otherwise>
						</xsl:choose>
						</td>
						<td style="text-align: right;"><xsl:value-of select="request/@requestTime"/></td>
						<td>
							 <xsl:attribute name="class">
								<xsl:choose>
								    <xsl:when test="starts-with(@testStatus,'exception')">
									<xsl:text>xunit-exception</xsl:text>
								    </xsl:when>
								    <xsl:when test="starts-with(@testStatus,'assertionFailure')">
									<xsl:text>xunit-assert</xsl:text>
								    </xsl:when>
								    <xsl:when test="starts-with(@testStatus,'assertionException') or starts-with(@testStatus,'setupException') or starts-with(@testStatus,'teardownException')">
									<xsl:text>xunit-testfail</xsl:text>
								    </xsl:when>
									<xsl:when test="name()='notest'">
										<xsl:text>xunit-notest</xsl:text>
									</xsl:when>
								    <xsl:otherwise>
									    <xsl:text>xunit-success</xsl:text>
								    </xsl:otherwise>   
								</xsl:choose> 
						    </xsl:attribute>
						
							<xsl:choose>
								<xsl:when test="name()='notest'">
									<xsl:text>notest</xsl:text>
								</xsl:when>
							    <xsl:otherwise>
							    	<xsl:if test="@testStatus='success'">
							    		<xsl:attribute name="class">xunit-success</xsl:attribute>
							    	</xsl:if>
								    <xsl:value-of select="@testStatus"/>
							    </xsl:otherwise>   
							</xsl:choose>
						</td>
						<td>
							<xsl:if test="name()='test'">
							<xsl:variable name="pos" select="1+count(preceding-sibling::test)"/>
							<input type="button" value="exec" onclick="parent.location='?index={$pos}'" class="btn"/>
							</xsl:if>
						</td>
					</tr>
					<xsl:choose>
					    <xsl:when test="starts-with(@testStatus,'assertionFailure')">
							<xsl:for-each select="assert/*[@assertStatus='false']">
								<tr class="{$class}">
									<td/><td></td><td colspan="3"><b><xsl:value-of select="name()"/></b> <xsl:if test="not(@assertExpected)">: <xsl:value-of select="."/></xsl:if></td>
								</tr>
								<xsl:if test="@assertExpected or @assertReceived">
								<tr  class="{$class}">
								<td/><td></td><td colspan="3">
									<xsl:if test="@assertExpected">
										<div>Expected: <xsl:value-of select="@assertExpected"/></div>
									</xsl:if>
									<xsl:if test="@assertReceived">
										<div>Received: <xsl:value-of select="@assertReceived"/></div>
									</xsl:if>
								</td>
								</tr>
								</xsl:if>
								
							</xsl:for-each>	
					    </xsl:when>
					  </xsl:choose>
			</xsl:for-each>
			</table>
			</td>
			</tr>
			</table>
		</div>
    </xsl:template>
	
	<xsl:template match="results">
		<table>
		<tr class="vis-title">
			<td>Date</td>
			<td>Total</td>
			<td>Run</td>
			<td>Success</td>
			<td>Exceptions</td>
			<td>Assert</td>
			<td>Execution Time</td>
			<td>Total Test Time</td>
		</tr>
		<tr style="background-image: none;">
				<xsl:attribute name="class">
					<xsl:choose>
					    <xsl:when test="testFailException+testFailAssert &gt; 0">
							<xsl:text>xunit-exception</xsl:text>
					    </xsl:when>
					    <xsl:otherwise>
							<xsl:text>xunit-success</xsl:text>
					    </xsl:otherwise>
					</xsl:choose>
				   </xsl:attribute>
			<td><xsl:value-of select="testDate"/></td>
			<td align="center"><xsl:value-of select="testTotal"/></td>
			<td align="center"><xsl:value-of select="testRun"/></td>
			<td align="center"><xsl:value-of select="testSuccess"/></td>
			<td align="center"><xsl:value-of select="testFailException"/></td>
			<td align="center"><xsl:value-of select="testFailAssert"/></td>
			<td align="center"><xsl:value-of select="testExecutionTime"/></td>
			<td align="center"><xsl:value-of select="testTotalTime"/></td>
		</tr>
		</table>
		<xsl:apply-templates/>
	</xsl:template>
		
	<xsl:template match="text()"/>
	
</xsl:stylesheet>
