<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:nk="http://netkernel.org" xmlns:xrl="http://netkernel.org/xrl">
	<xsl:output method="xml" />

	<xsl:template match="/spaces">
		<div>
			<xrl:include>
				 <xrl:identifier>active:controlPanelInfoBox</xrl:identifier>
				 <xrl:argument name="content">
				   <div>
             <p>
               The XUnit Test Manager allows a user to run installed unit tests.
             </p>
             <p>
               The NetKernel XUnit unit test framework is a comprehensive and extensible
               ROC testing system that provides unit test execution and result reporting
               along with user defined assertions and tests.
             </p>
				   <p>
             Installed test modules expose their test suites and are displayed in the
             test manager's main panel.
             Documentation on unit testing, assertions and test extensions are covered
             in the
				   <a href="/book/view/book:mod:test:book/doc:mod:test:guide:title">XUnit documentation</a>.
           </p>
             <p>
               In the display below, each test space
               is represented by one line.
               All of those tests may be run as a unit by
               clicking the button labeled &quot;exec&quot;.
               If the button labeled &quot;view&quot; is clicked,
               a display is presented with the full listing of the unit
               tests within a test suite.
             </p>
				   </div>
				 </xrl:argument>
			</xrl:include> 
			<table class="data-table">
				<tr class="oddrow">
					<td/>
					<td>
						<div><b>All Tests</b></div>
						<div>run all available installed tests</div>
					</td>
					<td>
						<input class="btn" type="button" value="xml" onclick="parent.location='/test/exec/xml/all'"/>
					</td>
					<td>
						<input class="btn" type="button" value="all" onclick="parent.location='/test/exec/html/all'"/>
					</td>
				</tr>
				<xsl:apply-templates select="space">
					<xsl:sort select="id" />
				</xsl:apply-templates>
			</table>
		</div>
	</xsl:template>

	<xsl:template match="space">
		<xsl:if test="tests/test">
			<xsl:for-each select="tests/test">
				<xsl:variable name="count" select="count(preceding::test)"/>
				<xsl:variable name="class">
					<xsl:choose>
						<xsl:when test="position() mod 2 = 0">
							<xsl:text>highrow</xsl:text>
						</xsl:when>
						<xsl:otherwise>
							<xsl:text>highrow</xsl:text>
						</xsl:otherwise>  
					</xsl:choose>
				</xsl:variable>
				<xsl:if test="position()=1">
					<tr><td colspan="4" style="padding: 10px;"></td></tr>
				</xsl:if>
				<tr class="{$class}">
				<xsl:choose>
					<xsl:when test="position()=1">
						<td style="font-weight: bold;">
							<xsl:value-of select="../../id" />
							<br />
							<xsl:value-of select="../../version" />
						</td>
					</xsl:when>
					<xsl:otherwise>
						<td/>
					</xsl:otherwise>
				</xsl:choose>
				<xsl:call-template name="test"/> 
				</tr>
			</xsl:for-each>
		</xsl:if>
	</xsl:template>

	<xsl:template name="test">
		<td>
			<div>
				<b>
					<xsl:value-of select="name" />
				</b>
			</div>
			<div class="subtitle">
				<xsl:value-of select="desc" />
			</div>
		</td>
		<td>
			<input class="btn" type="button" value="view" onclick="parent.location='/test/view/html/{id}'"/>
		</td>
		<td>
			<input class="btn" type="button" value="exec" onclick="parent.location='/test/exec/html/{id}'"/>
		</td>
	</xsl:template>

	<xsl:template match="text()" />

</xsl:stylesheet>
