package org.ten60.netkernel.test.endpoint;

import java.io.*;
import java.math.BigInteger;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Pattern;

import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.nkf.impl.NKFResponseReadOnlyImpl;
import org.netkernel.layer0.urii.SimpleIdentifierImpl;
import org.netkernel.layer0.util.RequestBuilder;
import org.netkernel.layer0.util.XMLReadable;
import org.netkernel.layer0.util.XMLUtils;
import org.netkernel.layer0.util.RequestBuilder.Arguments;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.impl.RequestScopeLevelImpl;
import org.netkernel.scheduler.IInternalResponseMeta;
import org.netkernel.urii.ISpaceWithIdentity;
import org.netkernel.urii.impl.Version;
import org.netkernel.xml.xda.DOMXDA;
import org.netkernel.xml.xda.IXDAIterator;
import org.ten60.netkernel.test.representation.TestEnginePreBuiltTestList;
import org.w3c.dom.*;


public class  TestEngineEndpoint extends StandardAccessorImpl
{	public static String TESTLIST_RESULTS_ROOT="/results";
	public static String TESTLIST_RESULTS_HEADER="/testlist"+TESTLIST_RESULTS_ROOT;
	
	/**
	 * Date format pattern RFC 1123
	 */
	 public static final String PATTERN_RFC1123 = "EEE, dd MMM yyyy HH:mm:ss zzz";
	 private static SimpleDateFormat mDateFormat=new SimpleDateFormat(PATTERN_RFC1123);
	 
	 private ISpaceWithIdentity mCallbackSpace;
	
	public TestEngineEndpoint()
	{	super();
		this.declareSourceRepresentation(Document.class);
		this.declareThreadSafe();
	}
	
	@Override
	protected void postCommission(INKFRequestContext aContext) throws Exception
	{	mCallbackSpace=aContext.getKernelContext().getKernel().getSpace(new SimpleIdentifierImpl("urn:org:netkernel:test:assert:callback"),null,null);
	}

	public void onSource(INKFRequestContext context) throws Exception
	{	context.logFormatted(INKFLocale.LEVEL_INFO, "MSG_TEST_STARTUP", new Object[]   { context.getThisRequest().getIdentifier() }  );
		String spaceArg=context.getThisRequest().getArgumentValue("space");
		String versionArg=context.getThisRequest().getArgumentValue("version");
		SimpleIdentifierImpl spaceID=new SimpleIdentifierImpl(spaceArg);
		Version version=new Version(versionArg);
		ISpaceWithIdentity space=context.getKernelContext().getKernel().getSpace(spaceID, version,version);
		
		long time=System.currentTimeMillis();
		//Source pre-built master test list
		context.setCWU(null);
		INKFRequest req=context.createRequest("active:testList");
		req.addArgument("space","arg:space");
		req.addArgument("version","arg:version");
		req.addArgument("tests","arg:tests");
		req.setRepresentationClass(TestEnginePreBuiltTestList.class);
		TestEnginePreBuiltTestList testList=(TestEnginePreBuiltTestList)context.issueRequest(req);
		
		//context.setCWU(context.getThisRequest().getArgumentValue("tests"));
		DOMXDA tl=testList.getMutableTestList();
				
		//Execute Tests
		int index=0;
		if(context.getThisRequest().argumentExists("index"))
		{	index=Integer.parseInt(context.getThisRequest().getArgumentValue("index"));
		}
		Object result=null;
		result=executeTests(tl, context, space, index);
		time=System.currentTimeMillis()-time;
		if(index==0)
		{	tl=(DOMXDA)result;
			tl.setText(TESTLIST_RESULTS_HEADER+"/space", spaceArg);
			tl.setText(TESTLIST_RESULTS_HEADER+"/version",versionArg);
			tl.setText(TESTLIST_RESULTS_HEADER+"/uri", context.getThisRequest().getArgumentValue("tests"));
			tl.setText(TESTLIST_RESULTS_HEADER+"/testTotalTime", time+"");
			tl.setText(TESTLIST_RESULTS_HEADER+"/testDate", mDateFormat.format(new Date()));
			result=tl.getDocument();
		}
		INKFResponse resp=context.createResponseFrom(result);
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
	}

	/**
	 *Execute each unique test.  Perform assertion tests on the result.
	 */
	private Object executeTests(DOMXDA tl, INKFRequestContext helper,  ISpaceWithIdentity space, int index) throws Exception
	{	IXDAIterator it=tl.iterator("/descendant::test");
		int testCounter=0,testRun=0, testFailedException=0, testFailedAssert=0;
		long testExecutionTime=0;
		//Clear the cache before each test cycle.
		helper.getKernelContext().getKernel().getRepresentationCache().clear();
		ClassLoader testEngineClassLoader=Thread.currentThread().getContextClassLoader();
		while(it.hasNext())
		{	it.next();
		
			XMLReadable test = new XMLReadable(((DOMXDA)it).getRoot());
			
			testCounter++;
			if(index==0 || (index==testCounter))
			{	INKFResponseReadOnly result=null;
				String resultMessage="skipped";
				long time=0;
				INKFRequest req=null;

				//Any setup needed?
				List<Element> setupNodes=(List<Element>)(List)test.getNodes("setup");
				if (setupNodes.size()>0)
				{	Element setupRequest=setupNodes.get(0);
					//Call setup...
					long setuptime=System.currentTimeMillis();
					try
					{	
						req=prepareRequest2(helper,setupRequest,space,null);
						helper.logFormatted(INKFLocale.LEVEL_INFO, "MSG_TEST_SETUP", new Object[]   {	req.toString()  }  );
						
						setuptime=System.currentTimeMillis();
						helper.issueRequest(req);
						setuptime=System.currentTimeMillis()-setuptime;
						it.appendPath("setup","@setupTime", setuptime+"");
					}
					catch(Exception t)
					{	String reqToString;
						if (req==null)
						{	reqToString="??";
						}
						else
						{	reqToString=req.toString();
						}
						
						helper.logFormatted(INKFLocale.LEVEL_WARNING, "EX_TEST_SETUP", new Object[]{ reqToString , t.toString() }	);
						resultMessage="setupException";
					}
					
				}
				
				if(!resultMessage.equals("setupException"))
				{	//Perform the test request					
					req=null;
					try
					{	time=System.currentTimeMillis();
						testRun++;
					
						List<Element> testNodes=(List<Element>)(List)test.getNodes("request");
						Element setupRequest=testNodes.get(0);
						req=prepareRequest2(helper,setupRequest,space,null);						
						
						//Execute Test
						time=System.currentTimeMillis();
						result=helper.issueRequestForResponse(req);
						//result=helper.getKernelContext().issueRootRequestForResponse(req);
						time=System.currentTimeMillis()-time;
						it.appendPath("request","@requestTime", time+"");
						testExecutionTime+=time;
						resultMessage="success";
						//Log the test outcome
						String msg=null;
						if(it.isTrue("@name"))
						{	msg=it.getText("@name", false);							
						}
						else
						{	msg=req.getIdentifier().toString();					
						}
						helper.logFormatted(INKFLocale.LEVEL_INFO, "MSG_TEST_COMPLETE", new Object[]  { msg, resultMessage, time }	);
						//Perform Assertion Tests
						int assertions=0;
						try
						{	req=null;
							IXDAIterator it2=it.iterator("assert/*");
							
							boolean assertResult=false;
							boolean hasAsserts=false;
							boolean hasAssertFailures=false;
							String assertExpected=null, assertReceived=null;
							while(it2.hasNext())
							{	it2.next();
								hasAsserts=true;
								assertions++;
								long asserttime=System.currentTimeMillis();
								String type=it2.eval("name()");
								Boolean ba=null;
								Node literalRoot = XMLUtils.getFirstChildElement(((DOMXDA)it2).getRoot());
								Object assertValue=null;
								if (literalRoot==null)
								{	assertValue=it2.getText(".", true);
									String value=(String)assertValue;
								
									//Try built-in asserts								
									if (type.equals("uri"))
									{	INKFRequest areq=helper.createRequest(value);
										areq.addArgumentByValue("result",result.getRepresentation());
										areq.setRepresentationClass(Boolean.class);
										ba=(Boolean)helper.issueRequest(areq);
										assertResult=ba.booleanValue();
									}
									else if (type.equals("maxTime"))
									{	assertResult=time<=Long.parseLong(value);
										if(!assertResult)
										{	assertExpected=value;
											assertReceived=Long.toString(time);
										}
									}
									else if (type.equals("minTime"))
									{	assertResult=time>=Long.parseLong(value);
										if(!assertResult)
										{	assertExpected=value;
											assertReceived=Long.toString(time);
										}
									}
									else if (type.equals("class"))
									{	Class c=this.getClassFromSpace(value,space);  
										assertResult=c.isInstance(result.getRepresentation());
										if(!assertResult)
										{	assertExpected=value;
											assertReceived=result.getRepresentation().getClass().getName();
										}
									}
									else if (type.equals("mimetype"))
									{	String mimetype=result.getMimeType();
										assertResult=value.equals(mimetype);
										if(!assertResult)
										{	assertExpected=value;
											assertReceived=mimetype;
										}
									}
									else if (type.equals("expired"))
									{	assertResult=result.isExpired();
									}
									else if (type.equals("notExpired"))
									{	assertResult=!result.isExpired();
									}
									else if (type.equals("true"))
									{	assertResult = ((Boolean)result.getRepresentation()).booleanValue();
									}
									else if (type.equals("false"))
									{	assertResult = !((Boolean)result.getRepresentation()).booleanValue();
									}
									else if (type.equals("null"))
									{	assertResult=result.getRepresentation()==null;
									}
									else if (type.equals("notNull"))
									{	assertResult=result.getRepresentation()!=null;
									}
									else if (type.equals("stringEquals"))
									{	Object rep=result.getRepresentation();
										if(rep==null)
										{	throw new Exception("Representation is null cannot assert stringEquals");										
										}
										assertResult=rep.toString().equals(value);
										if(!assertResult)
										{	assertExpected=value;
											assertReceived=rep.toString();
										}
									}
									else if (type.equals("int"))
									{	Object rep=result.getRepresentation();
										if(rep==null)
										{	throw new Exception("Representation is null cannot assert int");										
										}
										if(rep instanceof Integer)
										{
											Integer r=(Integer)rep;
											assertResult=Integer.parseInt(value)==r.intValue();
											if(!assertResult)
											{	assertExpected=value;
												assertReceived=Integer.toString(r.intValue());
											}
										}
										else if(rep instanceof Long)
										{
											Long r=(Long)rep;
											assertResult=Long.parseLong(value)==r.longValue();
											if(!assertResult)
											{	assertExpected=value;
												assertReceived=Long.toString(r.longValue());
											}
										}
										else if(rep instanceof BigInteger)
										{
											BigInteger r=(BigInteger)rep;
											assertResult= new BigInteger(value).equals(r);
											if(!assertResult)
											{	assertExpected=value;
												assertReceived=r.toString();
											}
										}
										else
										{	throw new Exception("Representation is not int, long or BigInteger so cannot assert int");
										}
									}
									else if (type.equals("float"))
									{	Object rep=result.getRepresentation();
										if(rep==null)
										{	throw new Exception("Representation is null cannot assert float");										
										}
										if(rep instanceof Float)
										{
											Float r=(Float)rep;
											assertResult=Float.parseFloat(value)==r.floatValue();
											if(!assertResult)
											{	assertExpected=value;
												assertReceived=Float.toString(r.floatValue());
											}
										}
										if(rep instanceof Double)
										{
											Double r=(Double)rep;
											assertResult=Double.parseDouble(value)==r.doubleValue();
											if(!assertResult)
											{	assertExpected=value;
												assertReceived=Double.toString(r.doubleValue());
											}
										}
									}
									else if (type.equals("regex"))
									{	Object rep=result.getRepresentation();
										if(rep==null)
										{	throw new Exception("Representation is null cannot assert regex");										
										}
										String reps=rep.toString();
										Pattern p=Pattern.compile(value, Pattern.DOTALL|Pattern.MULTILINE);
										assertResult=p.matcher(reps).matches();
										if(!assertResult)
										{	assertReceived=reps;
										}
									}
									//----------------------------------------------------
									//Built-in metadata tests
									//----------------------------------------------------
									else if (type.equals("minTotalCost"))
									{	assertResult=getInternalResponseMeta(result).getTotalCost()>=Integer.parseInt(value);
										if(!assertResult)
										{	assertExpected=value;
											assertReceived=Integer.toString(getInternalResponseMeta(result).getTotalCost());
										}
									}
									else if (type.equals("maxTotalCost"))
									{	assertResult=getInternalResponseMeta(result).getTotalCost()<=Integer.parseInt(value);
										if(!assertResult)
										{	assertExpected=value;
											assertReceived=Integer.toString(getInternalResponseMeta(result).getTotalCost());
										}
									}
									else if (type.equals("minLocalCost"))
									{	assertResult=getInternalResponseMeta(result).getLocalCost()>=Integer.parseInt(value);
										if(!assertResult)
										{	assertExpected=value;
											assertReceived=Integer.toString(getInternalResponseMeta(result).getLocalCost());
										}
									}
									else if (type.equals("maxLocalCost"))
									{	assertResult=getInternalResponseMeta(result).getLocalCost()<=Integer.parseInt(value);
										if(!assertResult)
										{	assertExpected=value;
											assertReceived=Integer.toString(getInternalResponseMeta(result).getLocalCost());
										}
									}
									else if (type.equals("headerExists"))
									{	String name;
										String classname="java.lang.Object";
										if(value.indexOf(",") > 0)
										{	String[] split=value.split(",");
											name=split[0];
											classname=split[1];
										}
										else
										{	name=value;											
										}
										
										if(result.hasHeader(name))
										{	Object header=result.getHeader(name);
											Class c=this.getClassFromSpace(classname,space);  
											assertResult=c.isInstance(header);
											if(!assertResult)
											{	assertExpected=name;
												assertReceived=header.getClass().getName();
											}
										}
										else assertResult=false;
									}
									else if (type.equals("scope"))
									{	NKFResponseReadOnlyImpl resp=(NKFResponseReadOnlyImpl)result;
										int depth=resp.getKernelResponse().getRequest().getRequestScope().getDepth();
										int scope=depth-getInternalResponseMeta(result).getUnresolvedScope();
										assertResult=scope==Integer.parseInt(value);
										if(!assertResult)
										{	assertExpected=value;
											assertReceived=Integer.toString(scope);
										}
										if(helper.shouldLog(INKFLocale.LEVEL_FINE))
										{	helper.logFormatted(INKFLocale.LEVEL_FINE, "MSG_ASSERT_SCOPE", new Object[]{	scope, value }	);
										}
									}
									else
									{	/*User assert with with String Argument*/
										IXDAIterator xdai= tl.iterator("/descendant::assertDefinition[@name='"+type+"']");
										if(xdai.hasNext())
										{	xdai.next();
											Element assertNode=(Element)((DOMXDA)xdai).getRoot();
											Arguments args=new Arguments();
											args.addArgument("arg:test:tagRef", value);
											args.addArgumentByValue("arg:test:tagValue", value);
											args.addArgumentByValue("arg:test:result", result.getRepresentation());
											args.addArgumentByValue("arg:test:response",result);
											
											req=prepareRequest2(helper, assertNode, space, args);
										
											//req=prepareRequest(helper,xdai, space, result, value);
											req.setRepresentationClass(Boolean.class);
											assertResult=(Boolean)helper.issueRequest(req);
											if(!assertResult)
											{
												//Source Expected and Received...
												try
												{
													IRequestScopeLevel scope=RequestScopeLevelImpl.createRootScopeLevel(mCallbackSpace);
													req=helper.createRequest("active:assert/Expected");
													req.setRequestScope(scope);
													req.setVerb(INKFRequestReadOnly.VERB_EXISTS);
													if((Boolean)helper.issueRequest(req))
													{
														req=helper.createRequest("active:assert/Expected");
														req.setRequestScope(scope);
														req.setRepresentationClass(java.lang.String.class);
														assertExpected=(String)helper.issueRequest(req);
														//Delete any state
														req=helper.createRequest("active:assert/Expected");
														req.setRequestScope(scope);
														req.setVerb(INKFRequestReadOnly.VERB_DELETE);
														helper.issueRequest(req);

													}
													req=helper.createRequest("active:assert/Received");
													req.setRequestScope(scope);
													req.setVerb(INKFRequestReadOnly.VERB_EXISTS);
													if((Boolean)helper.issueRequest(req))
													{
														req=helper.createRequest("active:assert/Received");
														req.setRequestScope(scope);
														req.setRepresentationClass(java.lang.String.class);
														assertReceived=(String)helper.issueRequest(req);
														//Delete any state
														req=helper.createRequest("active:assert/Received");
														req.setRequestScope(scope);
														req.setVerb(INKFRequestReadOnly.VERB_DELETE);
														helper.issueRequest(req);

													}
												}
												catch(Exception er)
												{	//Do nothing - shouldn't be possible but must protect assert Exception path 
												}
											}
											
										}
										else
										{	helper.logFormatted(INKFLocale.LEVEL_WARNING, "EX_ASSERT_UNKNOWN", new Object[]{	type }	);
											throw new Exception("Unknown assert: "+type);
										}
									}
								}
								else
								{	/*User assert with Literal Argument*/
									Document d=XMLUtils.newDocument();
									Node n=d.importNode(literalRoot, true);
									d.appendChild(n);
									Object literal[]=RequestBuilder.parseLiteral(n, space.getClassLoader(), helper);
									assertValue=literal[0];
									
									IXDAIterator xdai= tl.iterator("/descendant::assertDefinition[@name='"+type+"']");
									if(xdai.hasNext())
									{	xdai.next();
									
										Element assertNode=(Element)((DOMXDA)xdai).getRoot();
										
										Arguments args=new Arguments();
										args.addArgumentByValue("arg:test:tagValue", assertValue);
										args.addArgumentByValue("arg:test:result", result.getRepresentation());
										args.addArgumentByValue("arg:test:response",result);
										
										req=prepareRequest2(helper, assertNode, space, args);
									
										//req=prepareRequest(helper,xdai, space, result, assertValue);
										req.setRepresentationClass(Boolean.class);
										assertResult=(Boolean)helper.issueRequest(req);
										
										if(!assertResult)
										{
											//Source Expected and Received...
											try
											{
												IRequestScopeLevel scope=RequestScopeLevelImpl.createRootScopeLevel(mCallbackSpace);
												req=helper.createRequest("active:assert/Expected");
												req.setRequestScope(scope);
												req.setVerb(INKFRequestReadOnly.VERB_EXISTS);
												if((Boolean)helper.issueRequest(req))
												{
													req=helper.createRequest("active:assert/Expected");
													req.setRequestScope(scope);
													req.setRepresentationClass(java.lang.String.class);
													assertExpected=(String)helper.issueRequest(req);
													//Delete any state
													req=helper.createRequest("active:assert/Expected");
													req.setRequestScope(scope);
													req.setVerb(INKFRequestReadOnly.VERB_DELETE);
													helper.issueRequest(req);
												}
												req=helper.createRequest("active:assert/Received");
												req.setRequestScope(scope);
												req.setVerb(INKFRequestReadOnly.VERB_EXISTS);
												if((Boolean)helper.issueRequest(req))
												{
													req=helper.createRequest("active:assert/Received");
													req.setRequestScope(scope);
													req.setRepresentationClass(java.lang.String.class);
													assertReceived=(String)helper.issueRequest(req);
													//Delete any state
													req=helper.createRequest("active:assert/Received");
													req.setRequestScope(scope);
													req.setVerb(INKFRequestReadOnly.VERB_DELETE);
													helper.issueRequest(req);
												}
											}
											catch(Exception er)
											{	//Do nothing - shouldn't be possible but must protect assert Exception path 
											}
										}
									}
									else
									{	helper.logFormatted(INKFLocale.LEVEL_WARNING, "EX_ASSERT_UNKNOWN", new Object[]{	type }	);	
										throw new Exception("Unknown assert: "+type);
									}
								}
								hasAssertFailures |= !assertResult;
								asserttime=System.currentTimeMillis()-asserttime;
								it2.appendPath(".","@assertStatus", assertResult ? "true" : "false");
								it2.appendPath(".","@assertTime", asserttime+"");
								if(assertExpected!=null)
								{	it2.appendPath(".","@assertExpected", assertExpected);
									assertExpected=null;
								}
								if(assertReceived!=null)
								{	it2.appendPath(".","@assertReceived", assertReceived);
									assertReceived=null;
								}
								//Log the assert outcome separately from the entire assertResult:
								String assertMessage = (assertResult ? "success" : "failed");
								if(assertResult)
								{	helper.logFormatted(INKFLocale.LEVEL_INFO, "MSG_ASSERT_COMPLETE", new Object[]  { type, assertValue, assertMessage}  );
								}
								else
								{	helper.logFormatted(INKFLocale.LEVEL_WARNING, "MSG_ASSERT_FAILED", new Object[]  { type, assertValue, assertMessage}  );
								}
							}
							if(hasAsserts && hasAssertFailures)
							{	testFailedAssert++;
								resultMessage="assertionFailure";
							}
						}
						catch(Throwable t)
						{	if(!(t instanceof ThreadDeath))
							{
								String reqToString;
								if (req==null)
								{	reqToString="??";
								}
								else
								{	reqToString=req.toString();
								}
								
								helper.logFormatted(INKFLocale.LEVEL_WARNING, "EX_ASSERT", new Object[]{	reqToString , t.toString() }	);
								if(helper.shouldLog(INKFLocale.LEVEL_FINE))
								{	StringWriter sw=new StringWriter();
									t.printStackTrace(new PrintWriter(sw));	
									helper.logFormatted(INKFLocale.LEVEL_FINE, "EX_ASSERT", new Object[]{	reqToString ,  sw.toString()}	);
								}
								resultMessage="assertionException";
								testFailedAssert++;
							}
						}
					}
					catch(Throwable t)
					{	if(!(t instanceof ThreadDeath))
						{
							String reqToString;
							if (req==null)
							{	reqToString="??";
							}
							else
							{	reqToString=req.toString();
							}
							
							//Switch back to testEngineClassLoader
							Thread.currentThread().setContextClassLoader(testEngineClassLoader);
							if(index==testCounter)
							{	if(t instanceof Exception)
								throw (Exception)t;
							}
							time=System.currentTimeMillis()-time;
							IXDAIterator it2=it.iterator("assert/exception");
							if (it2.hasNext())
							{	it2.next();
								if(it2.isTrue("./text()"))
								{
									String value=it2.getText(".", true);
									if (t instanceof NKFException)
									{	String id = ((NKFException)t).getDeepestId();
										if (!value.equals(id))
										{	testFailedException++;
											resultMessage="exception";
											helper.logFormatted(INKFLocale.LEVEL_WARNING, "EX_TEST", new Object[]{	reqToString , t.toString() }	);
										}
										else
										{	
											resultMessage="success";
											//Log the assert outcome
											helper.logFormatted(INKFLocale.LEVEL_INFO, "MSG_TEST_COMPLETE", new Object[]  { reqToString, resultMessage, "N/A" }	);
										}
									}
									else
									{	testFailedException++;
										resultMessage="exception";
										helper.logFormatted(INKFLocale.LEVEL_WARNING, "EX_TEST", new Object[]{	reqToString , t.toString() }	);
										if(helper.shouldLog(INKFLocale.LEVEL_FINE))
										{	StringWriter sw=new StringWriter();
											t.printStackTrace(new PrintWriter(sw));	
											helper.logFormatted(INKFLocale.LEVEL_FINE, "EX_TEST", new Object[]{	reqToString ,  sw.toString()}	);
										}
									}
								}
								else
								{	resultMessage="success";
									//Log the assert outcome
									helper.logFormatted(INKFLocale.LEVEL_INFO, "MSG_TEST_COMPLETE", new Object[]  { reqToString, resultMessage, "N/A" }	);
								}
							}
							else
							{	testFailedException++;
								resultMessage="exception";
								helper.logFormatted(INKFLocale.LEVEL_WARNING, "EX_TEST", new Object[]{	reqToString , t.toString() }	);
								if(helper.shouldLog(INKFLocale.LEVEL_FINE))
								{	StringWriter sw=new StringWriter();
									t.printStackTrace(new PrintWriter(sw));	
									helper.logFormatted(INKFLocale.LEVEL_FINE, "EX_TEST", new Object[]{	reqToString ,  sw.toString()}	);
								}
							}
							if(!it.isTrue("request/@requestTime"))
							{	it.appendPath("request", "@requestTime", time+"");
							}
						}
					}
					//Any teardown ?
					List<Element> teardownNodes=(List<Element>)(List)test.getNodes("teardown");
					if (teardownNodes.size()>0)
					{	Element teardownRequest=teardownNodes.get(0);
						long teardowntime=System.currentTimeMillis();
						try
						{	req=prepareRequest2(helper,teardownRequest,space,null);
							teardowntime=System.currentTimeMillis();
							helper.issueRequest(req);
							teardowntime=System.currentTimeMillis()-teardowntime;
							it.appendPath("teardown","@teardownTime", teardowntime+"");
							helper.logFormatted(INKFLocale.LEVEL_INFO, "MSG_TEST_TEARDOWN", new Object[]  { req.toString() }	);
						}
						catch(Exception t)
						{	String reqToString;
							if (req==null)
							{	reqToString="??";
							}
							else
							{	reqToString=req.toString();
							}
							helper.logFormatted(INKFLocale.LEVEL_WARNING, "EX_TEST_TEARDOWN", new Object[]{ reqToString , t.toString() }	);
							resultMessage="teardownException";
						}
					}

				}			
				it.appendPath(".", "@testStatus", resultMessage);
				if(index>0)
				{	return result; 					
				}
			}
		}
		//Clear the cache after each test cycle.
		helper.getKernelContext().getKernel().getRepresentationCache().clear();
		//Set Stats
		tl.setText(TESTLIST_RESULTS_HEADER+"/testTotal",testCounter+"");
		tl.setText(TESTLIST_RESULTS_HEADER+"/testRun",testRun+"");
		tl.setText(TESTLIST_RESULTS_HEADER+"/testSuccess",testRun-testFailedException-testFailedAssert+"");
		tl.setText(TESTLIST_RESULTS_HEADER+"/testFailException", testFailedException+"");
		tl.setText(TESTLIST_RESULTS_HEADER+"/testFailAssert", testFailedAssert+"");
		tl.setText(TESTLIST_RESULTS_HEADER+"/testExecutionTime", testExecutionTime+"");
		return tl;
	}
	
	private INKFRequest prepareRequest2(INKFRequestContext aContext, Element aRequest, ISpaceWithIdentity aSpace, Arguments aValues) throws Exception
	{
		IRequestScopeLevel originalScope=aContext.getKernelContext().getRequestScope();
		IRequestScopeLevel scope=RequestScopeLevelImpl.createRootScopeLevel(mCallbackSpace);
		scope=RequestScopeLevelImpl.createScopeLevel(aSpace, scope, false);
		
		aContext.getKernelContext().setRequestScope(scope);
		RequestBuilder rb=new RequestBuilder(aRequest,aContext.getKernelContext().getKernel().getLogger());
		ClassLoader cl=aSpace.getClassLoader();
		if (cl==null)
		{	cl=Thread.currentThread().getContextClassLoader();
		}
		INKFRequest req=rb.buildRequest(aContext, aValues, cl);
		//req.setHeader(INKFRequest.HEADER_FORGET_DEPENDENCIES, Boolean.TRUE);
		req.setRequestScope(scope);
		
		aContext.getKernelContext().setRequestScope(originalScope);
		return req;
	}
	
	
	/*
	private INKFRequest prepareRequest(INKFRequestContext context, IXDAReadOnly xda, ISpaceWithIdentity space, Object result, Object assertval) throws Exception
	{	
		INKFRequest req;
		IRequestScopeLevel scope=RequestScopeLevelImpl.createRootScopeLevel(space);
		//IRequestScopeLevel scope=RequestScopeLevelImpl.createOrphanedRootScopeLevel(space, context.getKernelContext().getThisKernelRequest().getRequestScope());
		
		int v=INKFRequestReadOnly.VERB_SOURCE;
		if(xda.isTrue("verb"))
		{	String verb=xda.getText("verb",true);
			v=RequestFactory.parseVerb(verb);
			if(v==0)  throw context.createFormattedException("EX_UNKNOWN_VERB","MSG_UNKNOWN_VERB", null,null,verb);
		}
		if(v==INKFRequestReadOnly.VERB_TRANSREPRESENT )
		{	String id=SimpleIdentifierImpl.getUnique("literal:var").toString();
			req=context.createRequest(id);
			//Transrept Representation
			String trepid=xda.getText("operand", true);
			INKFRequest req2=context.createRequest(trepid);
			req2.setRequestScope(scope);
			Object trep=context.issueRequest(req2);
			req.addPrimaryArgument(trep);
		}
		else
		{	if(xda.isTrue("base"))
			{	String base=xda.getText("base", true);
				req=context.createRequest(base);
			}
			else
			{	String endpoint=xda.getText("endpoint",true);
				context.setCWU(null);
				req=context.createRequestToEndpoint(endpoint);
			}
		}
		if(v==INKFRequestReadOnly.VERB_SINK)
		{	//Sink Representation
			String sinkrepid=xda.getText("operand", true);
			INKFRequest req2=context.createRequest(sinkrepid);
			req2.setRequestScope(scope);
			Object sinkrep=context.issueRequest(req2);
			req.addPrimaryArgument(sinkrep);
		}
		req.setVerb(v);
		Class c=Object.class;
		if(xda.isTrue("class"))
		{	String claz=xda.getText("class", true);
			c=getClassFromSpace(claz, space);
		}
		req.setRepresentationClass(c);
		IXDAReadOnlyIterator it=xda.readOnlyIterator("argument");
		while(it.hasNext())
		{	it.next();
			String name=it.getText("@name", true);
			Node literalRoot = XMLUtils.getFirstChildElement(((DOMXDA)it).getRoot());
			Object value;
			if (literalRoot==null)
			{	value=it.getText(".",true);
				if(assertval!=null)
				{	String val=(String)value;
					if(val.equals("arg:test:tagValue"))
					{	req.addArgumentByValue(name,assertval);
					}
					else if(val.equals("arg:test:tagRef"))
					{	if(assertval instanceof String)
						{	req.addArgument(name,(String)assertval);
						}
						else throw new IllegalArgumentException("Assert value is a literal and must be a string for arg:test:tagRef declaration");
					}
					else if(val.equals("arg:test:result"))
					{	req.addArgumentByValue(name, ((INKFResponseReadOnly)result).getRepresentation());						
					}
					else if(val.equals("arg:test:response"))
					{	req.addArgumentByValue(name, result);						
					}
					else
					{	req.addArgument(name,val);
					}
				}
				else
				{	req.addArgument(name,(String)value);
				}
			}
			else
			{	Document d=XMLUtils.newDocument();
				Node n=d.importNode(literalRoot, true);
				d.appendChild(n);
				req.addArgumentByValue(name,d);
			}
		}
		req.setRequestScope(scope);
		return req;
	}
	*/
	
	private Class getClassFromSpace(String classname,  ISpaceWithIdentity space) throws Exception
	{	ClassLoader cl=space.getClassLoader();
		if(cl==null)
		{	cl=Thread.currentThread().getContextClassLoader();
		}
		 return cl.loadClass(classname);
	}
	
	private IInternalResponseMeta getInternalResponseMeta(INKFResponseReadOnly response)
	{		NKFResponseReadOnlyImpl resp=(NKFResponseReadOnlyImpl)response;
			return (IInternalResponseMeta)resp.getKernelResponse().getMeta();
	}
}
