package org.ten60.netkernel.test.endpoint;

import java.util.HashMap;

import org.netkernel.layer0.nkf.INKFLocale;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer0.urii.SimpleIdentifierImpl;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.impl.RequestScopeLevelImpl;
import org.netkernel.urii.ISpaceWithIdentity;
import org.netkernel.urii.impl.Version;
import org.netkernel.util.Utils;
import org.netkernel.xml.util.XMLUtils;
import org.netkernel.xml.xda.DOMXDA;
import org.netkernel.xml.xda.IXDAIterator;
import org.ten60.netkernel.test.representation.TestEnginePreBuiltTestList;
import org.w3c.dom.Document;

public class TestListBuilderAccessor extends StandardAccessorImpl
{	
	public TestListBuilderAccessor()
	{	this.declareThreadSafe();
		this.declareName("TestListPreBuilder");
		this.declareDescription("Builds an immutable pre-built testlist by recursively sourcing all test groups");
	}

	public void onSource(INKFRequestContext context) throws Exception
	{	//Source master test list
		SimpleIdentifierImpl spaceID=new SimpleIdentifierImpl(context.getThisRequest().getArgumentValue("space"));
		Version version=new Version(context.getThisRequest().getArgumentValue("version"));
		ISpaceWithIdentity space=context.getKernelContext().getKernel().getSpace(spaceID, version,version);
		IRequestScopeLevel scope=RequestScopeLevelImpl.createOrphanedRootScopeLevel(space, context.getKernelContext().getThisKernelRequest().getRequestScope());
		INKFRequest req=context.createRequest(context.getThisRequest().getArgumentValue("tests"));
		req.setRequestScope(scope);
		req.setRepresentationClass(IReadableBinaryStreamRepresentation.class);
		IReadableBinaryStreamRepresentation testListStream=(IReadableBinaryStreamRepresentation)context.issueRequest(req);
		Document testList=(Document)context.transrept(testListStream, Document.class);
		//Validate test list
		validateTestList(testList, context, context.getThisRequest().getIdentifier(), scope);		
		DOMXDA tl=new DOMXDA(testList, true);
		
		//Prepare Result Headers
		DOMXDA header=new DOMXDA(XMLUtils.newDocument());
		header.appendPath("/","results",null);
		header.appendPath(TestEngineEndpoint.TESTLIST_RESULTS_ROOT, "space", null);
		header.appendPath(TestEngineEndpoint.TESTLIST_RESULTS_ROOT, "version", null);
		header.appendPath(TestEngineEndpoint.TESTLIST_RESULTS_ROOT, "uri", null);
		header.appendPath(TestEngineEndpoint.TESTLIST_RESULTS_ROOT, "testTotal", null);
		header.appendPath(TestEngineEndpoint.TESTLIST_RESULTS_ROOT, "testRun", null);
		header.appendPath(TestEngineEndpoint.TESTLIST_RESULTS_ROOT, "testSuccess", null);
		header.appendPath(TestEngineEndpoint.TESTLIST_RESULTS_ROOT, "testFailException", null);
		header.appendPath(TestEngineEndpoint.TESTLIST_RESULTS_ROOT, "testFailAssert", null);
		header.appendPath(TestEngineEndpoint.TESTLIST_RESULTS_ROOT, "testExecutionTime", null);
		header.appendPath(TestEngineEndpoint.TESTLIST_RESULTS_ROOT, "testTotalTime",null);
		header.appendPath(TestEngineEndpoint.TESTLIST_RESULTS_ROOT, "testDate",null);
		tl.insertBefore(header,"/","/testlist/*[1]");
		
		//Recursively build test list
		HashMap groups=new HashMap(10);
		tl=recurseTestList(tl, context, groups, scope);
		if(context.getThisRequest().argumentExists("index"))
		{	String index=context.getThisRequest().getArgumentValue("index");
			tl.appendPath("//test[position() = "+index+"]","@testTarget", "true");
			tl.delete("//test[not (@testTarget='true')]");
			//tl.delete("//test/@testtarget");
		}
		context.createResponseFrom(new TestEnginePreBuiltTestList(tl.getDocument()));
	}

	/**
	 * Recursively generate test tree by pulling in any further testlists
	 * Detect and prevent circularities by only including each unique test group once.
	 */
	private DOMXDA recurseTestList(DOMXDA tl, INKFRequestContext context, HashMap groups, IRequestScopeLevel scope) throws Exception
	{	IXDAIterator it=tl.iterator("descendant::import");
		while(it.hasNext())
		{	it.next();
			String uri=it.getText(".", true);
			if(groups.get(uri)==null)
			{	groups.put(uri,new Boolean(true));
				try
				{	INKFRequest req=context.createRequest(uri);
					req.setRequestScope(scope);
					IReadableBinaryStreamRepresentation testListStream=(IReadableBinaryStreamRepresentation)context.issueRequest(req);
					Document tlxa=(Document)context.transrept(testListStream, Document.class);
					validateTestList(tlxa,context, uri, scope);
					DOMXDA tl2=new DOMXDA(tlxa,true);
					tl2=recurseTestList(tl2, context, groups, scope);
					it.append(tl2, "/", ".");
				}
				catch(NKFException e)
				{	if(e.getDeepestId().equals("Request Resolution Failure"))
					{	context.logFormatted(INKFLocale.LEVEL_WARNING,"EX_MISSING_TESTLIST", uri, Utils.throwableToString(e) );					
					}
					else throw e;
				}
			}
			else
			{	it.appendPath(".", "/duplicate",null);
			}
		}
		return tl;
	}
	
	
	private void validateTestList(Document testlist, INKFRequestContext helper, String uri, IRequestScopeLevel scope) throws Exception
	{	//To Be Done - need RNG validator accessor and testlist schema		
	}
	
}
